package com.ehsure.ehsure_location_map.location.api;

import android.app.Application;

import com.ehsure.ehsure_location_map.location.utils.NetWorkUtil;

import java.io.File;
import java.util.concurrent.TimeUnit;

import okhttp3.Cache;
import okhttp3.CacheControl;
import okhttp3.Interceptor;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory;
import retrofit2.converter.fastjson.FastJsonConverterFactory;

public class ApiManage {

    private static final String BASE_URL = "https://apis.map.qq.com/ws/place/v1/";

    private static final Interceptor REWRITE_CACHE_CONTROL_INTERCEPTOR = chain -> {
        Request request = chain.request();
        request = request.newBuilder().build();
        Response response = chain.proceed(request);
        return response.newBuilder().removeHeader("Pragma").build();
    };

    private static HttpLoggingInterceptor logging = new HttpLoggingInterceptor().setLevel(HttpLoggingInterceptor.Level.BODY);

    private static ApiManage apiManage;
//    private static File httpCacheDirectory = new File(AppContext.getInstance().getCacheDir(), "textCache");
    private static int cacheSize = 10 * 1024 * 1024; // 10 MiB
//    private static Cache cache = new Cache(httpCacheDirectory, cacheSize);
    private static int DEFAULT_TIMEOUT = 15;

    //设置缓存空间及大小，设置拦截器
    private static OkHttpClient client = new OkHttpClient.Builder()
            .addNetworkInterceptor(REWRITE_CACHE_CONTROL_INTERCEPTOR)
            .addInterceptor(REWRITE_CACHE_CONTROL_INTERCEPTOR)
            .addInterceptor(logging)
//            .cache(cache)
            .connectTimeout(DEFAULT_TIMEOUT, TimeUnit.SECONDS) //超时
            .writeTimeout(DEFAULT_TIMEOUT, TimeUnit.SECONDS)
            .readTimeout(DEFAULT_TIMEOUT, TimeUnit.SECONDS)
            .build();


    private final Object monitor = new Object();

    public static ApiManage getInstance() {
        if (apiManage == null) {
            synchronized (ApiManage.class) {
                if (apiManage == null) {
                    apiManage = new ApiManage();
                }
            }
        }
        return apiManage;
    }

    private RetrofitRequest request;

    RetrofitRequest getRetrofitService() {
        if (request == null) {
            synchronized (monitor) {
                if (request == null) {
                    request = new Retrofit.Builder()
                            .baseUrl(BASE_URL)
                            .addCallAdapterFactory(RxJava2CallAdapterFactory.create())
                            .client(client)
                            .addConverterFactory(FastJsonConverterFactory.create())
                            .build().create(RetrofitRequest.class);
                }
            }
        }
        return request;
    }

}
