package com.ehsure.ehsure_location_map.location.base;

import android.Manifest;
import android.os.Build;
import android.os.Bundle;

import androidx.annotation.Nullable;

import com.ehsure.ehsure_location_map.location.base.delegate.IActivity;
import com.ehsure.ehsure_location_map.location.base.delegate.IPresenter;
import com.ehsure.ehsure_location_map.location.di.component.ActivityComponent;
import com.ehsure.ehsure_location_map.location.di.component.DaggerActivityComponent;
import com.ehsure.ehsure_location_map.location.di.component.DaggerAppComponent;
import com.ehsure.ehsure_location_map.location.di.module.ActivityModule;
import com.ehsure.ehsure_location_map.location.di.module.AppModule;
import com.trello.navi2.component.support.NaviAppCompatActivity;

import pub.devrel.easypermissions.AfterPermissionGranted;
import pub.devrel.easypermissions.EasyPermissions;

public abstract class BaseActivity<P extends IPresenter> extends NaviAppCompatActivity implements IActivity {

    protected ActivityComponent mActivityComponent;
    protected P mPresenter;

    public ActivityComponent getActivityComponent() {
        return mActivityComponent;
    }

    public abstract void initInjector();

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initActivityComponent();
        if (provideContentViewId() != 0) {
            setContentView(provideContentViewId());//布局
        }
        initData();
    }

    private void initActivityComponent() {
        mActivityComponent = DaggerActivityComponent.builder()
                .appComponent(DaggerAppComponent.builder()
                        .appModule(new AppModule(this))
                        .build())
                .activityModule(new ActivityModule(this))
                .build();
        initInjector();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mPresenter != null) mPresenter.onDestroy();
        this.mPresenter = null;
    }

    private long lastClickTime;

    protected boolean isFastDoubleClick() {
        long time = System.currentTimeMillis();
        long timeD = time - lastClickTime;
        if (0 < timeD && timeD < 1000) {
            return true;
        }
        lastClickTime = time;
        return false;
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @AfterPermissionGranted(1)
    protected void requirePermission() {
        String[] permissions = {
                Manifest.permission.ACCESS_COARSE_LOCATION,
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.READ_PHONE_STATE,
                Manifest.permission.WRITE_EXTERNAL_STORAGE,
                Manifest.permission.READ_EXTERNAL_STORAGE
        };
        String[] permissionsForQ = new String[0];
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            permissionsForQ = new String[]{
                    Manifest.permission.ACCESS_COARSE_LOCATION,
                    Manifest.permission.ACCESS_FINE_LOCATION,
                    Manifest.permission.ACCESS_BACKGROUND_LOCATION, //target为Q时，动态请求后台定位权限
                    Manifest.permission.READ_PHONE_STATE
            };
        }
        if (Build.VERSION.SDK_INT >= 29 ? EasyPermissions.hasPermissions(this, permissionsForQ) :
                EasyPermissions.hasPermissions(this, permissions)) {
        } else {
            EasyPermissions.requestPermissions(this, "需要获取操作权限",
                    1, Build.VERSION.SDK_INT >= 29 ? permissionsForQ : permissions);
        }
    }

}
