package com.ehsure.ehsure_location_map.location.presenter.impl;

import android.annotation.SuppressLint;
import android.app.Activity;

import com.ehsure.ehsure_location_map.location.api.ApiService;
import com.ehsure.ehsure_location_map.location.presenter.LocationPresenter;
import com.ehsure.ehsure_location_map.location.presenter.base.BasePresenterImpl;
import com.ehsure.ehsure_location_map.location.ui.IView.LocationView;
import com.trello.rxlifecycle2.android.ActivityEvent;

import javax.inject.Inject;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class LocationPresenterImpl extends BasePresenterImpl<LocationView> implements LocationPresenter {

    private Activity mActivity;

    @Inject
    LocationPresenterImpl(Activity activity) {
        mActivity = activity;
    }

    @SuppressLint("CheckResult")
    @Override
    public void searchPoi(double lat, double lon, int page_index, int page_size) {
        String boundary = "nearby(" + lat + "," + lon + ",200)";
        mView.showLoading();
        new ApiService().searchPoi(boundary,page_index,page_size)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .compose(getLifecycleProvider(mActivity).bindUntilEvent(ActivityEvent.DESTROY))
                .subscribe(data -> {
                    mView.hideLoading();
                    if (data.getStatus() == 0) {
                        mView.setPoiModel(data);
                    } else {
                        mView.showMessage(data.getMessage());
                    }
                }, this::loadError);
    }

    @SuppressLint("CheckResult")
    @Override
    public void searchPoiWithKeyword(String keyword, double lat, double lon, int page_index, int page_size) {
        String boundary = "nearby(" + lat + "," + lon + ",500)";
        mView.showLoading();
        new ApiService().searchPoiWithKeyword(keyword,boundary,page_index,page_size)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .compose(getLifecycleProvider(mActivity).bindUntilEvent(ActivityEvent.DESTROY))
                .subscribe(data -> {
                    mView.hideLoading();
                    if (data.getStatus() == 0) {
                        mView.setPoiModel(data);
                    } else {
                        mView.showMessage(data.getMessage());
                    }
                }, this::loadError);
    }

    private void loadError(Throwable throwable) {
        throwable.printStackTrace();
        mView.hideLoading();
        mView.showMessage(throwable.getMessage());
    }

}
