package com.ehsure.ehsure_location_map.location.ui.activity;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Looper;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.ehsure.ehsure_location_map.R;
import com.ehsure.ehsure_location_map.location.base.BaseActivity;
import com.ehsure.ehsure_location_map.location.listener.OnLoadMoreListener;
import com.ehsure.ehsure_location_map.location.models.PoiModel;
import com.ehsure.ehsure_location_map.location.presenter.LocationPresenter;
import com.ehsure.ehsure_location_map.location.presenter.impl.LocationPresenterImpl;
import com.ehsure.ehsure_location_map.location.ui.IView.LocationView;
import com.ehsure.ehsure_location_map.location.utils.Constants;
import com.ehsure.ehsure_location_map.location.widget.SearchEditText;
import com.google.android.material.snackbar.Snackbar;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.tencent.map.geolocation.TencentLocation;
import com.tencent.map.geolocation.TencentLocationListener;
import com.tencent.map.geolocation.TencentLocationManager;
import com.tencent.mapsdk.raster.model.BitmapDescriptorFactory;
import com.tencent.mapsdk.raster.model.CircleOptions;
import com.tencent.mapsdk.raster.model.LatLng;
import com.tencent.mapsdk.raster.model.Marker;
import com.tencent.mapsdk.raster.model.MarkerOptions;
import com.tencent.tencentmap.mapsdk.map.MapView;
import com.tencent.tencentmap.mapsdk.map.TencentMap;
import com.tencent.tencentmap.mapsdk.map.UiSettings;

import java.util.ArrayList;
import java.util.List;

import javax.inject.Inject;

public class LocationSelectActivity extends BaseActivity<LocationPresenter> implements LocationView, TencentLocationListener {

    @Inject
    LocationPresenterImpl mPresenter;
    MapView mapView;
    RecyclerView recyclerView;
    SwipeRefreshLayout swipeRefreshLayout;
    SearchEditText setSearch;
    private Marker marker;
    private TencentMap tencentMap;
    private double mLat;
    private double mLon;
    private int pageNum = 1;
    //    private int authPositionDeviation;
    private static final int PAGE_SIZE = 20;
    private ListAdapter adapter;
    private List<PoiModel.DataModel> modelList = new ArrayList<>();
    PoiModel.DataModel selectedDataModel = new PoiModel.DataModel();
    private TencentLocationManager locationManager;
    private String keyword;
    private TextView tvWaiting;

    @Override
    public void initInjector() {
        mActivityComponent.inject(this);
        mPresenter.attachView(this);
    }

    @Override
    public int provideContentViewId() {
        return R.layout.activity_location_select;
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            View decorView = getWindow().getDecorView();
            int option = View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_LAYOUT_STABLE;
            decorView.setSystemUiVisibility(option);
            getWindow().setStatusBarColor(getResources().getColor(android.R.color.transparent));
        } else {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            //恢复状态栏白色字体
            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_VISIBLE);
            //设置状态栏黑色字体
//            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
        }
        if (Build.VERSION.SDK_INT >= 23) requirePermission();
    }

    @Override
    public void initData() {
        mapView = findViewById(R.id.map);
        tvWaiting = findViewById(R.id.tv_waiting);
        recyclerView = findViewById(R.id.recyclerView);
        swipeRefreshLayout = findViewById(R.id.swipeRefreshLayout);
        setSearch = findViewById(R.id.set_search);
        tencentMap = mapView.getMap();
        tencentMap.setZoom(17);
//        authPositionDeviation = getIntent().getIntExtra("distance", 1000);
        tencentMap.setOnMapClickListener(latLng -> {
            double latitude = latLng.getLatitude();
            double longitude = latLng.getLongitude();
//            double distance = CommonUtil.getDistance(mLat, mLon, latitude, longitude);
//            if (distance > authPositionDeviation) {
//                Snackbar.make(mapView, "所选地点超出可选范围", Snackbar.LENGTH_SHORT).show();
//                return;
//            }
            selectLocation(latitude, longitude);
        });
        UiSettings uiSettings = mapView.getUiSettings();
        uiSettings.setLogoScale(0.5f);
        LinearLayoutManager layoutManager;
        recyclerView.setLayoutManager(layoutManager = new LinearLayoutManager(this));
        recyclerView.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL));
        recyclerView.setHasFixedSize(true);
        recyclerView.setItemAnimator(new DefaultItemAnimator());
        recyclerView.setAdapter(adapter = new ListAdapter());
        recyclerView.addOnScrollListener(new OnLoadMoreListener(layoutManager) {
            @Override
            public void onLoadMore() {
                if (modelList.size() < PAGE_SIZE * (pageNum - 1)) {
                    return;
                }
                if (TextUtils.isEmpty(keyword))
                    mPresenter.searchPoi(mLat, mLon, pageNum, PAGE_SIZE);
                else
                    mPresenter.searchPoiWithKeyword(keyword, mLat, mLon, pageNum, PAGE_SIZE);
            }
        });
        setSearch.setSearchClickListener(() -> {
            keyword = setSearch.getText().toString();
            if (TextUtils.isEmpty(keyword)) {
                Toast.makeText(this, "请输入地点", Toast.LENGTH_SHORT).show();
                return;
            }
            pageNum = 1;
            modelList.clear();
            mPresenter.searchPoiWithKeyword(keyword, mLat, mLon, pageNum, PAGE_SIZE);
        });
        ImageView imageView = findViewById(R.id.iv_show_location);
        imageView.setOnClickListener(v -> {
            locationManager = TencentLocationManager.getInstance(LocationSelectActivity.this);
            locationManager.requestSingleFreshLocation(null,
                    LocationSelectActivity.this, Looper.getMainLooper());
        });
        Button btnConfirm = findViewById(R.id.btn_confirm);
        btnConfirm.setOnClickListener(v -> {
            if (TextUtils.isEmpty(selectedDataModel.getId())) {
                showMessage("正在定位，请稍候");
                return;
            }
            GsonBuilder gsonBuilder = new GsonBuilder();
            gsonBuilder.setPrettyPrinting();
            Gson gson = gsonBuilder.create();
            String json = gson.toJson(selectedDataModel);
            Intent retIntent = new Intent();
            retIntent.putExtra(Constants.RESULT_SCAN_DATA_KEY,json);
            setResult(RESULT_OK, retIntent);
            finish();
        });
        locationManager = TencentLocationManager.getInstance(this);
        locationManager.requestSingleFreshLocation(null,
                LocationSelectActivity.this, Looper.getMainLooper());
    }

    @Override
    public void setPoiModel(PoiModel poiModel) {
        final List<PoiModel.DataModel> data = poiModel.getData();
        if (data == null || data.size() == 0) {
            Snackbar.make(recyclerView, "没有更多数据了", Snackbar.LENGTH_SHORT).show();
            return;
        }
        modelList.addAll(data);
        pageNum++;
        adapter.notifyDataSetChanged();
    }

    @Override
    public void showLoading() {
        swipeRefreshLayout.setEnabled(true);
        swipeRefreshLayout.setRefreshing(true);
    }

    @Override
    public void hideLoading() {
        swipeRefreshLayout.setRefreshing(false);
        swipeRefreshLayout.setEnabled(false);
    }

    @Override
    public void showMessage(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onLocationChanged(TencentLocation tencentLocation, int i, String s) {
        if (i == TencentLocation.ERROR_OK) {
            mapView.setVisibility(View.VISIBLE);
            tvWaiting.setVisibility(View.INVISIBLE);
            mLat = tencentLocation.getLatitude();
            mLon = tencentLocation.getLongitude();
            LatLng latLng = new LatLng(mLat, mLon);
            tencentMap.addCircle(new CircleOptions().
                    center(latLng).
//                    radius(authPositionDeviation).
        fillColor(getResources().getColor(R.color.transparent)).
                            strokeColor(getResources().getColor(R.color.blue)).
                            strokeWidth(3));
            selectLocation(mLat, mLon);
        }
    }

    @Override
    public void onStatusUpdate(String s, int i, String s1) {

    }

    class ListAdapter extends RecyclerView.Adapter<ListAdapter.ItemViewHolder> {

        @NonNull
        @Override
        public ItemViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
            View view = LayoutInflater.from(LocationSelectActivity.this).inflate(R.layout.item_location_select, parent, false);
            return new ItemViewHolder(view);
        }

        @Override
        public void onBindViewHolder(ItemViewHolder holder, int position) {
            PoiModel.DataModel dataModel = modelList.get(position);
            holder.tvName.setText(dataModel.getTitle());
            holder.tvAddress.setText(dataModel.getAddress());
            if (TextUtils.isEmpty(selectedDataModel.getId()) && position == 0) {
                holder.ivSelected.setVisibility(View.VISIBLE);
                selectedDataModel = dataModel;
            } else {
                if (TextUtils.equals(selectedDataModel.getId(), dataModel.getId())) {
                    holder.ivSelected.setVisibility(View.VISIBLE);
                } else
                    holder.ivSelected.setVisibility(View.GONE);
            }
            holder.itemView.setOnClickListener(v -> {
                PoiModel.DataModel.LocationModel location = dataModel.getLocation();
//                double distance = CommonUtil.getDistance(mLat, mLon, location.getLat(), location.getLng());
//                if (distance > authPositionDeviation) {
//                    Snackbar.make(mapView, "所选地点超出可选范围", Snackbar.LENGTH_SHORT).show();
//                    return;
//                }
                selectedDataModel = dataModel;
                animateToPosition(location.getLat(), location.getLng());
                notifyDataSetChanged();
            });
        }

        @Override
        public int getItemCount() {
            return modelList.size();
        }

        class ItemViewHolder extends RecyclerView.ViewHolder {

            TextView tvName;
            TextView tvAddress;
            ImageView ivSelected;

            ItemViewHolder(View view) {
                super(view);
                tvName = view.findViewById(R.id.tv_name);
                tvAddress = view.findViewById(R.id.tv_address);
                ivSelected = view.findViewById(R.id.iv_selected);
            }
        }
    }

    private void animateToPosition(double lat, double lon) {
        LatLng latLng = new LatLng(lat, lon);
        if (marker == null) {
            marker = tencentMap.addMarker(new MarkerOptions().
                    position(latLng).
                    icon(BitmapDescriptorFactory.fromResource(R.mipmap.red_location)).
                    anchor(0.5f, 0.5f));
        }
        marker.setPosition(latLng);
        tencentMap.animateTo(latLng);
    }

    private void selectLocation(double lat, double lon) {
        selectedDataModel.setId(null);
        animateToPosition(lat, lon);
        modelList.clear();
        pageNum = 1;
        mPresenter.searchPoi(lat, lon, pageNum, PAGE_SIZE);
    }

}
